/*
 * This code is sample code, provided as-is, and we make no
 * warranties as to its correctness or suitablity for
 * any purpose.
 * IT HAS NOT BEEN TESTED FOR CORRECTNESS IN PRODUCTION SYSTEMS
 *
 * We hope that it's useful to you.  Enjoy.
 * Copyright 2006-10 LearningPatterns Inc.
 */

package com.javatunes.persist;

import javax.persistence.EntityManager;
import javax.persistence.EntityManagerFactory;
import javax.persistence.EntityTransaction;
import javax.persistence.Persistence;

/**
 * Basic JPA helper class, handles EntityManagerFactory, EntityManager and Transaction.
 * <p>
 * Uses a static initializer for the initial EntityManagerFactory creation
 * and holds EntityManager and Transactions in thread local variables.
 *
 */
public class JpaUtil {

    private static final String PU_NAME = "javatunes";
    private static EntityManagerFactory emf;
    private static final ThreadLocal<EntityManager> threadEntityManager = new ThreadLocal<EntityManager>();
    private static final ThreadLocal<EntityTransaction> threadTransaction = new ThreadLocal<EntityTransaction>();

	// Create the initial EntityManagerFactory
    static {
        emf = Persistence.createEntityManagerFactory(PU_NAME);
    }

    /**
     * Returns the EntityManagerFactory used for this static class.
     *
     * @return EntityManagerFactory
     */
    public static EntityManagerFactory getEntityManagerFactory() {
        return emf;
    }

    /**
     * Retrieves the current EntityManager local to the thread.
     * <p/>
     * If no EntityManager is open, opens a new EntityManager for the running thread.
     *
     * @return EntityManager
     */
    public static EntityManager getEntityManager() {
        EntityManager em = threadEntityManager.get();
        if (em == null) {
            System.out.println("Opening new EntityManager for this thread.");
            em = getEntityManagerFactory().createEntityManager();
        }
        threadEntityManager.set(em);
        return em;
    }
    
    /**
     * Closes the EntityManager local to the thread.
     */
    public static void closeEntityManager() {
        EntityManager em = threadEntityManager.get();
        threadEntityManager.set(null);
        if (em != null && em.isOpen()) {
            System.out.println("Closing EntityManager of this thread.");
            em.close();
        }
    }

    /**
     * Start a new database transaction.
     */
    public static void beginTransaction() {
        EntityTransaction tx = threadTransaction.get();
        if (tx == null) {
            System.out.println("Starting new database transaction in this thread.");
            tx = getEntityManager().getTransaction();
            threadTransaction.set(tx);
            tx.begin();
        }
    }

    /**
     * Commit the database transaction.
     */
    public static void commitTransaction() {
        EntityTransaction tx = threadTransaction.get();
        if (tx != null && tx.isActive()) {
            if (tx.getRollbackOnly()) {
                System.out.println("Rolling back database transaction of this thread.");
                rollbackTransaction();

            } else {  // end if
                System.out.println("Committing database transaction of this thread.");
                tx.commit();
            } // end else
        } // end if (tx != null ...)
        threadTransaction.set(null);
    } // end commitTranscation()

    /**
     * Rollback the database transaction.
     */
    public static void rollbackTransaction() {
        EntityTransaction tx = threadTransaction.get();
        try {
            threadTransaction.set(null);
            if (tx != null && tx.isActive()) {
                System.out.println("Tyring to rollback database transaction of this thread.");
                tx.rollback();
            } // end if
        } // end try
        finally {
            closeEntityManager();  // Assume persistence context no longer valid
        }
    } // end rollbackTransaction()
}
